/******************************************************************************
 * @file     main.c
 * @brief    Main program body
 * @version  V1.8.0
 * @date     31-Dec-2021
 *
 * @note
 * Copyright (C) 2021 Spintrol Limited Corporation. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include "spc1158.h"
#include <stdio.h>
#include <string.h>


ErrorStatus                 eErrorState;
uint8_t                     u8PREDRIDATA;                                                      /* Data read from PRE-DRIVER */
uint8_t                     u8PREDRIID                                    = 0;                 /* PRE-DRIVER mode ID */
uint32_t                    u32SSPBaundrate                               = 2000;              /* The baundrate of SSP communicating with High voltage mode */
uint32_t                    u32Timeout                                    = 500;               /* The number of times try to get PRE-DRIVER mode ID */




 /*************************************************************************************************************************
 *
 * @brief      This case show how to set the BUCK FREQ as a dithering FREQ. you can observe a dithering waveform with a
 *             osciloscope connecting to the SWG PAD of the chip.
 *
 *             In PFM mode, dithering FREQ is not easy to observe the dithering state of the waveform. So, there give a
 *             diagram about BUCK in PWM mode(but PWM mode is not recommanded in real production). The 'spread-spectrum
 *             amplitude' in BUCKCTL1 register change the 'change time'(showed in diagram).
 *
 *                                          |          |        |      |         |      |        |
 *                                          ||---      ||--     ||-    |-   -    |-     ||-      ||--
 *                                         ||    |    ||   |   ||  |  || | | |  || |   ||  |    ||   |
 *                                         |     |    |    |   |   |  |  | | |  |  |   |   |    |    |
 *                                         |     |    |    |   |   |  |  | | |  |  |   |   |    |    |
 *                                         |     ||   |    ||  |   || |  | | || |  ||  |   ||   |    |
 *                                                ||--      ||-     |-    -   |-    ||-     ||--
 *                                                |         |       |         |     |       |
 *                                               |             |
 *                                               |<-Change---->| 
 *                                               |    time     |
 *
 **************************************************************************************************************************/


int main()
{
  FLASH_WALLOW();
  FLASH_SetTiming(100000000);
  /* Disable flash write access after flash operation had done */
  FLASH_WDIS();

  CLOCK_InitWithRCO(CLOCK_HCLK_100MHZ);

  Delay_Init();

  /*
   * Init the UART
   *
   * 1.Set the GPIO34/35 as UART FUNC
   *
   * 2.Enable the UART CLK
   *
   * 3.Set the rest para
   */
  GPIO_SetPinChannel(GPIO_34,GPIO34_UART_TXD);
  GPIO_SetPinChannel(GPIO_35,GPIO35_UART_RXD);
  CLOCK_EnableModule(UART_MODULE);
  UART_Init(UART,38400);

  /* Init High Voltage part */
  eErrorState = HV_Init(u32SSPBaundrate, &u8PREDRIID, u32Timeout);
  if(eErrorState == ERROR)
  {
    printf("Init High Voltage FAIL\n");
    return 0;
  }
  else
  {
    printf("Init High Voltage SUCCESS[ID:%d]\n",u8PREDRIID);
  }

  /* Set BUCK FREQ as 600KHz and force BUCK into PWM mode */
  eErrorState = HV_ReadReg(HV_REG_BUCKCTL0, &u8PREDRIDATA);
  if(eErrorState == ERROR)
  {
    printf("Read HV_REG_BUCKCTL0 register FAIL[%02x]\n",u8PREDRIDATA);
    return 0;
  }

  eErrorState = HV_WriteReg(HV_REG_BUCKCTL0, ((u8PREDRIDATA & (~0xC4)) | BUCKCTL0_ALL_FRCPWM_FORCE | BUCKCTL0_ALL_FREQSEL_600_kHz));
  if(eErrorState == ERROR)
  {
    printf("Force BUCK into PWM mode and set FREQ as 600 KHz FAIL\n");
    return 0;
  }

  /* Enable spread and set spread spectrum amplitude */
  eErrorState = HV_ReadReg(HV_REG_BUCKCTL1, &u8PREDRIDATA);
  if(eErrorState == ERROR)
  {
    printf("Read HV_REG_BUCKCTL1 register FAIL[%02x]\n",u8PREDRIDATA);
    return 0;
  }

  eErrorState = HV_WriteReg(HV_REG_BUCKCTL1, ((u8PREDRIDATA & (~0xE0)) | BUCKCTL1_ALL_SSEN_ENABLE | BUCKCTL1_ALL_SSAMP_0_TO_511));
  if(eErrorState == ERROR)
  {
    printf("Enabel BUCK spread and set spread spectrum amplitude FAIL\n");
    return 0;
  }

  /* Release SPI interface IO resource, in order to enable these IO can be reused */
  /* Release SPI interface */
  EPWR_DisableComInterface();
  
  /* Enable EPWR Protect */
  EPWR_WDIS();

  while(1)
  {

  }
}
