/******************************************************************************
 * @file     main.c
 * @brief    Main program body
 * @version  V1.8.0
 * @date     31-Dec-2021
 *
 * @note
 * Copyright (C) 2021 Spintrol Limited Corporation. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include "spc1158.h"
#include <stdio.h>
#include <string.h>

/* This macro is used to get the PWM period with a specified frequency */
#define                     PWMPeriod(u32PWMFreqHz)           ((CLOCK_GetModuleClock(PWM_MODULE))/u32PWMFreqHz)/2;


#define                     PRE_DRIVER_PWM_FREQ                          1000                  /* 1kHz PWM */
#define                     PWM_DB_NS                                    1000                  /* 1us */
#define                     PWM_BlankWIN_Size                            100                   /* 100 TBCNT CLK */
#define                     PWM_Blank_Offset                             50                    /* 50 TBCNT CLK */
#define                     EPWR_TH_VAL_NS                               10                    /* 10 ns */
#define                     EPWR_WIN_VAL_NS                              10                    /* 10 ns */


ErrorStatus                 eErrorState;
uint32_t                    u32PWMPeriod;
uint8_t                     u8PREDRIDATA;                                                      /* Data read from PRE-DRIVER */
uint8_t                     u8PREDRIID                                    = 0;                 /* PRE-DRIVER mode ID */
uint32_t                    u32SSPBaundrate                               = 2000;           /* The baundrate of SSP communicating with High Voltage part */
uint32_t                    u32Timeout                                    = 500;               /* The number of times try to get PRE-DRIVER mode ID */


void PWMx_Set_Digiter_Compare(PWM_REGS * PWMx)
{
  /* 
   * The following step can is the coding style of Digital Compare sub-module structure in PWM module,
   * for more information, please refer to Figure 10-16 of SPC1158 TRM.
   */

  /* Step1. Set EPWRTZ0 as the DCAH input */
  PWM_EnableDCAHTripEvent(PWMx, DC_TRIP_EPWRTZ1);

  /* Step2. Trip DCAEVT0 event when DCAH is high(be carefull about the DCH and DCL polarity) */
  PWM_SetRawDCAEVT0(PWMx, DCH_HIGH_DCL_X);

  /*
   * Step3. Config the Event filter function.
   *
   * <1>.Configure DC event filter
   *
   * <2>.Input is DCAEVT0, apply blank window with original polarity
   *
   * <3>.Set the blank window position relative to TBCNT=0
   */
  PWM_SetDCFilter(PWMx, DCF_FROM_RAW_DCAEVT0, DCF_ALIGN_ON_ZERO);

  /* Set blank window size as 100 TBCLK and offset as 50 TBCLK */
  PWM_SetDCFilterBlankWindow(PWMx, PWM_BlankWIN_Size, PWM_Blank_Offset);

  /* Enbale PWM DC filter blank function */
  PWM_EnableDCFilterBlank(PWMx);

  /* 
   * Step4. Set the event sent to Event processing logic.
   *
   * Set the Filtered DCAEVT0 event or the Raw DCAEVT0 as the event which
   * will be sent to Event processing logic. 
   */
  PWM_SetDCAEVT0(PWMx, DCEVT_FILTERED);

  /* Step5. Enable the Event processing logic output EVT0 signal(DCAEVT0 int) to Trip-zone sub-module */
  PWM_EnableDCAEVT0TripInt(PWMx);

  /* Step blow is the configuration of Trip-zone sub-module, nothing to do with Digital compare sub-module */
  /* Set Trip-Zone as one-shot mode */
  PWM_SetOneShotTripEvent(PWMx, TRIP_EVENT_DCAEVT, TRIP_OUTPUT_LATCH);

  /* Set PWM output tri-state upon DCAEVT0 trip event */
  PWM_SetCHAOutputWhenTrip(PWMx, TZU_TRIP_DO_NOTHING |
                                 TZD_TRIP_DO_NOTHING |
                                 DCEVT0U_TRIP_AS_TRI_STATE |
                                 DCEVT0D_TRIP_AS_TRI_STATE |
                                 DCEVT1U_TRIP_DO_NOTHING |
                                 DCEVT1D_TRIP_DO_NOTHING);

  PWM_SetCHBOutputWhenTrip(PWMx, TZU_TRIP_DO_NOTHING |
                                 TZD_TRIP_DO_NOTHING |
                                 DCEVT0U_TRIP_AS_TRI_STATE |
                                 DCEVT0D_TRIP_AS_TRI_STATE |
                                 DCEVT1U_TRIP_DO_NOTHING |
                                 DCEVT1D_TRIP_DO_NOTHING);
}


 /*************************************************************************************************************************
 *
 * @brief      This case show how to use over voltage monitor function, TZ event will be triggered and crossponding PWM
 *             channel waveform will stop immediately and the infomation in INT function will print when Vds too high event
 *             happended.
 *
 *                                                        SPC1158 CHIP
 *                            --------------------------------------------------------------------------------
 *                           |               MCU                                                              |
 *                           |    -------------------------------------                     --------------    |
 *                           |   |      -------------                  |                   |              |   |
 *                           |   |     |  --------   |TZ signal  ------|      TZ signal    |              |   |
 *                           |   |     | |  DC SUB|<-|----------|      |<------------------X              |   |
 *                           |   |     | |        |  |          |      |                   |              |   |
 *                           |   |     | |  MODE  |  |          | EPWR |                   | High voltage |   |
 *                           |   |     |  --------   |           ------|    PWM waveform   |  analog part |   |
 *                           |   |     |     PWM     |-----------------x------------------>|              |   |
 *                           |   |      -------------                  |                   |              |   |
 *                           |    -------------------------------------                     --------------    |
 *                            --------------------------------------------------------------------------------
 *             X : Chip internal PAD
 *             
 *             Key Point:
 *                       <1> Two step need for TZ function ��Enable TZ function on the EPWR side, Enable TZ function on the
 *                           MCU side.
 *
 **************************************************************************************************************************/


int main()
{
  FLASH_WALLOW();
  FLASH_SetTiming(100000000);
  /* Disable flash write access after flash operation had done */
  FLASH_WDIS();

  CLOCK_InitWithRCO(CLOCK_HCLK_100MHZ);

  Delay_Init();

  /*
   * Init the UART
   *
   * 1.Set the GPIO34/35 as UART FUNC
   *
   * 2.Enable the UART CLK
   *
   * 3.Set the rest para
   */
  GPIO_SetPinChannel(GPIO_34,GPIO34_UART_TXD);
  GPIO_SetPinChannel(GPIO_35,GPIO35_UART_RXD);
  CLOCK_EnableModule(UART_MODULE);
  UART_Init(UART,38400);

  /* Init High Voltage part */
  eErrorState = HV_Init(u32SSPBaundrate, &u8PREDRIID, u32Timeout);
  if(eErrorState == ERROR)
  {
    printf("Init High Voltage FAIL\n");
    return 0;
  }
  else
  {
    printf("Init High Voltage SUCCESS[ID:%d]\n",u8PREDRIID);
  }

  /* Set over voltage event as the trip-zone source and high valid */
  eErrorState = HV_WriteReg(HV_REG_TZCTL, (TZCTL_ALL_OVEN_ENABLE | TZCTL_ALL_POL_HIGH));
  if(eErrorState == ERROR)
  {
    printf("Write HV_REG_TZCTL register FAIL\n");
    return 0;
  }

  /* Set VDDG over voltage threshold as 15.2V */
  eErrorState = HV_ReadReg(HV_REG_BUCKCTL1, &u8PREDRIDATA);
  if(eErrorState == ERROR)
  {
    printf("Read HV_REG_BUCKCTL1 register FAIL[%02x]\n",u8PREDRIDATA);
    return 0;
  }

  eErrorState = HV_WriteReg(HV_REG_BUCKCTL1, (u8PREDRIDATA & (~0x3)) | BUCKCTL1_ALL_SELOV_15dot2_V);
  if(eErrorState == ERROR)
  {
    printf("Write HV_REG_BUCKCTL1 register FAIL\n");
    return 0;
  }
  /* Disable EPWR Protect */
  EPWR_WALLOW();

  /* Enbale EPWR module receive TZ signal from high voltage analog part(be aware EPWRTZ0 is invalid in SPC1158) */
  EPWR_EnableTripEvent(EPWR_TRIP_EVENT_TZ1);

  /* Set the input signal of TZ1 event as high */
  EPWR_SetTripEventInputActiveHigh(EPWR_TRIP_EVENT_TZ1);

  /* 
   * High voltage part TZ signal jump from one PAD(on High voltage side) to another side(on MCU side), this could
   * cause erroneous TZ assertion signal, so, another filter need to be set on the EPWR side(EPWR is a internal mode of MCU).
   */
  EPWR_SetFilterWindowTimeNs(EPWR_TRIP_EVENT_TZ1, EPWR_WIN_VAL_NS, EPWR_TH_VAL_NS);

  /* Select TZ output signal(was sent to MCU) type */
  EPWR_SetTripEventOutputType(EPWR_TRIP_EVENT_TZ1, EPWR_OUTPUT_FILTERED);

  /* Release SPI interface IO resource, in order to enable these IO can be reused */
  /* Release SPI interface */
  EPWR_DisableComInterface();
  
  /* Enable EPWR Protect */
  EPWR_WDIS();

  /* 
   * Init PWM0~5
   *
   * Warnning : 
   *           when PWM is used for PRE-DRIVER, the dead-band of the pair complementary waveform of one PWM channel
   *           must not equal zero, bigger than 600 is recommanded.
   */
  PWM_ComplementaryPairChannelInit(PWM5, PRE_DRIVER_PWM_FREQ, PWM_DB_NS);

  /* Set PWM5A output 75% duty waveform */
  u32PWMPeriod = PWMPeriod(PRE_DRIVER_PWM_FREQ);
  PWM_SetCMPA(PWM5, u32PWMPeriod / 4);

  /* Enable the TZ function on MCU side(each of six PWM is enable) */
  PWMx_Set_Digiter_Compare(PWM5);

  /* Select GPIO36/37 as the channel A/B output of PWM5 respectively */
  GPIO_SetPinChannel(GPIO_36,GPIO36_PWM5A);
  GPIO_SetPinChannel(GPIO_37,GPIO37_PWM5B);

  /* Start PWM module */
  PWM_RunCounter(PWM5);

  /* Enabel global INT for WPM TZ */
  NVIC_EnableIRQ(PWM5TZ_IRQn);

  while(1)
  {

  }
}

void PWM5TZ_IRQHandler()
{
  printf("PWM5 TZ INT happened\n");
}
